package cc.fascinated.bat.command;

import lombok.Getter;
import lombok.NonNull;
import lombok.Setter;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;

/**
 * A sub-command for a {@link BatCommand}.
 *
 * @author Braydon
 */
@Setter @Getter
public abstract class BatSubCommand implements IBatCommand {
    /**
     * The parent of this sub-command.
     */
    @NonNull private final BatCommand parent;

    /**
     * The snowflake of this sub=command, if it
     * was successfully registered on Discord.
     */
    private Long snowflake;

    /**
     * The full name of this command, including the
     * parent name, category, and sub-command name.
     */
    private String fullName;

    /**
     * The optional group name for this sub-command.
     */
    private final String groupName;

    /**
     * The data for this sub-command.
     */
    @NonNull private final SubcommandData commandData;

    public BatSubCommand(@NonNull BatCommand parent, @NonNull String name, @NonNull String description, @NonNull OptionData... options) {
        this(parent, null, name, description, options);
    }

    public BatSubCommand(@NonNull BatCommand parent, String groupName, @NonNull String name, @NonNull String description, @NonNull OptionData... options) {
        this.parent = parent;
        this.groupName = groupName;
        commandData = new SubcommandData(name, description).addOptions(options);
    }

    /**
     * Get the name of this command.
     *
     * @return the command name
     */
    @Override @NonNull
    public String getName() {
        return commandData.getName();
    }

    /**
     * Get the description of this command.
     *
     * @return the command description
     */
    @Override @NonNull
    public String getDescription() {
        return commandData.getDescription();
    }

    /**
     * Get the mentionable name of this command.
     *
     * @return the mentionable name
     */
    @Override @NonNull
    public final String getMentionable() {
        return "</%s:%s>".formatted(getFullName(), getSnowflake());
    }
}